@@ -141,45 +141,70 @@ enum {
  * The following section contains keys used by the GDM configuration files.
  * The key/value pairs defined in the GDM configuration files are considered
  * "stable" interface and should only change in ways that are backwards
- * compatible.
+ * compatible.  Please keep this in mind when changing GDM configuration.
  * 
  * Developers who add new configuration options should ensure that they do the
  * following:
  * 
+ * + Edit the config/gdm.conf file to include the default setting.
+ *
  * + Specify the same default in this file as in the config/gdm.conf.in file.
  *
- * + Update the val_hash and type_hash settings in gdm_config_init function
- *   in daemon/gdmconfig.c to add the new options.
+ * + Update the gdm_config_init function in daemon/gdmconfig.c to add the
+ *   new key.  Include some documentation about the new key, following the
+ *   style of existing comments.
  *
  * + Add any validation to the _gdm_set_value_string, _gdm_set_value_int,
- *   and/or _gdm_set_value_bool functions in gdmconfig.c, if needed.
- *
- * + The gui/gdmsetup.c program should be updated to support the new option
- *   unless there's a good reason not to.
+ *   and/or _gdm_set_value_bool functions (depending on the type of the new
+ *   key) in gdmconfig.c, if validation is needed.
  *
  * + If GDM_UPDATE_CONFIG should not respond to this configuration setting,
  *   update the update_config function in gdmconfig.c to return FALSE for
  *   this key.  Examples include changing the PidFile, ServAuthDir, or
- *   other values that GDM shouldn't change until it is restarted.  If 
+ *   other values that GDM should not change until it is restarted.  If 
  *   this is true, the next bullet can be ignored.
  *
  * + If the option should cause the greeter (gdmlogin/gdmgreeter) program to
  *   be updated immediately, make sure to update the appropriate 
- *   _gdm_set_value_* function in gdmconfig.c to cause a call to
- *   notify_displays_*() when this value is changed.  Supporting logic will
- *   also be needed in the gdm_slave_handle_notify function in slave.c.
- *   It should be simple to see how to do this from the other examples.
+ *   _gdm_set_value_* function in gdmconfig.c.  This function calls the
+ *   notify_displays_* function to call when this value is changed, so you
+ *   will need to add your new config value to the list of values sending
+ *   such notification.  Supporting logic will need to be added to
+ *   gdm_slave_handle_notify function in slave.c to process the notify.
+ *   It should be clear to see how to do this from the existing code.
+ *
+ * + Add the key to the gdm_read_config and gdm_reread_config functions in 
+ *   gui/gdmlogin.c, gui/gdmchooser.c, and gui/greeter/greeter.c
+ *   if the key is used by those programs.  Note that all GDM slaves load
+ *   all their configuration data between calls to gdmcomm_comm_bulk_start()
+ *   and gdmcomm_comm_bulk_stop().  This makes sure that the slave only uses
+ *   a single sockets connection to get all configuration data.  If a new
+ *   config value is read by a slave, make sure to load the key in this
+ *   code section for best performance.
+ *
+ * + The gui/gdmsetup.c program should be updated to support the new option
+ *   unless there's a good reason not to (like it is a configuration value
+ *   that only someone who really knows what they are doing should change).
+ * 
+ * + Currently GDM treats any key in the "gui" and "greeter" categories,
+ *   and security/PamStack as available for per-display configuration.  
+ *   If a key is appropriate for per-display configuration, and is not
+ *   in the "gui" or "greeter" categories, then it will need to be added
+ *   to the gdm_config_key_to_string_per_display function.  It may make
+ *   sense for some keys used by the daemon to be per-display so this
+ *   will need to be coded (refer to GDM_KEY_PAM_STACK for an example).
  *
  * + Update the docs/C/gdm.xml file to include information about the new
  *   option.  Include information about any other interfaces (such as 
  *   ENVIRONMENT variables) that may affect the configuration option.
+ *   Patches without documentation will not be accepted.
  *
  * Please do this work *before* submitting an patch.  Patches that are not
  * complete will not likely be accepted.
  */
 
 /* Configuration constants */
-#define GDM_KEY_CHOOSER "daemon/Chooser=" EXPANDED_LIBEXECDIR "/gdmchooser"
+#define GDM_KEY_CHOOSER "daemon/Chooser=" LIBEXECDIR "/gdmchooser"
 #define GDM_KEY_AUTOMATIC_LOGIN_ENABLE "daemon/AutomaticLoginEnable=false"
 #define GDM_KEY_AUTOMATIC_LOGIN "daemon/AutomaticLogin="
 /* The SDTLOGIN feature is Solaris specific, and causes the Xserver to be
@@ -191,27 +216,27 @@ enum {
 #else
 #define GDM_KEY_ALWAYS_RESTART_SERVER "daemon/AlwaysRestartServer=false"
 #endif
-#define GDM_KEY_GREETER "daemon/Greeter=" EXPANDED_LIBEXECDIR "/gdmlogin"
-#define GDM_KEY_REMOTE_GREETER "daemon/RemoteGreeter=" EXPANDED_LIBEXECDIR "/gdmlogin"
+#define GDM_KEY_GREETER "daemon/Greeter=" LIBEXECDIR "/gdmlogin"
+#define GDM_KEY_REMOTE_GREETER "daemon/RemoteGreeter=" LIBEXECDIR "/gdmlogin"
 #define GDM_KEY_ADD_GTK_MODULES "daemon/AddGtkModules=false"
 #define GDM_KEY_GTK_MODULES_LIST "daemon/GtkModulesList="
 #define GDM_KEY_GROUP "daemon/Group=gdm"
 #define GDM_KEY_HALT "daemon/HaltCommand=" HALT_COMMAND
-#define GDM_KEY_DISPLAY_INIT_DIR "daemon/DisplayInitDir=" EXPANDED_SYSCONFDIR "/gdm/Init"
+#define GDM_KEY_DISPLAY_INIT_DIR "daemon/DisplayInitDir=" GDMCONFDIR "/Init"
 #define GDM_KEY_KILL_INIT_CLIENTS "daemon/KillInitClients=true"
-#define GDM_KEY_LOG_DIR "daemon/LogDir=" EXPANDED_LOGDIR
+#define GDM_KEY_LOG_DIR "daemon/LogDir=" LOGDIR
 #define GDM_KEY_PATH "daemon/DefaultPath=" GDM_USER_PATH
-#define GDM_KEY_PID_FILE "daemon/PidFile=/var/run/gdm.pid"
-#define GDM_KEY_POSTSESSION "daemon/PostSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PostSession/"
-#define GDM_KEY_PRESESSION "daemon/PreSessionScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
-#define GDM_KEY_POSTLOGIN "daemon/PostLoginScriptDir=" EXPANDED_SYSCONFDIR "/gdm/PreSession/"
+#define GDM_KEY_PID_FILE "daemon/PidFile=" GDM_PID_FILE
+#define GDM_KEY_POSTSESSION "daemon/PostSessionScriptDir=" GDMCONFDIR "/PostSession/"
+#define GDM_KEY_PRESESSION "daemon/PreSessionScriptDir=" GDMCONFDIR "/PreSession/"
+#define GDM_KEY_POSTLOGIN "daemon/PostLoginScriptDir=" GDMCONFDIR "/PreSession/"
 #define GDM_KEY_FAILSAFE_XSERVER "daemon/FailsafeXServer="
-#define GDM_KEY_X_KEEPS_CRASHING "daemon/XKeepsCrashing=" EXPANDED_SYSCONFDIR "/gdm/XKeepsCrashing"
+#define GDM_KEY_X_KEEPS_CRASHING "daemon/XKeepsCrashing=" GDMCONFDIR "/XKeepsCrashing"
 #define GDM_KEY_REBOOT  "daemon/RebootCommand=" REBOOT_COMMAND
 #define GDM_KEY_ROOT_PATH "daemon/RootPath=/sbin:/usr/sbin:" GDM_USER_PATH
-#define GDM_KEY_SERV_AUTHDIR "daemon/ServAuthDir=" EXPANDED_AUTHDIR
-#define GDM_KEY_SESSION_DESKTOP_DIR "daemon/SessionDesktopDir=/etc/X11/sessions/:" EXPANDED_SYSCONFDIR "/dm/Sessions/:" EXPANDED_DATADIR "/gdm/BuiltInSessions/:" EXPANDED_DATADIR "/xsessions/"
-#define GDM_KEY_BASE_XSESSION "daemon/BaseXsession=" EXPANDED_SYSCONFDIR "/gdm/Xsession"
+#define GDM_KEY_SERV_AUTHDIR "daemon/ServAuthDir=" AUTHDIR
+#define GDM_KEY_SESSION_DESKTOP_DIR "daemon/SessionDesktopDir=/etc/X11/sessions/:" DMCONFDIR "/Sessions/:" DATADIR "/gdm/BuiltInSessions/:" DATADIR "/xsessions/"
+#define GDM_KEY_BASE_XSESSION "daemon/BaseXsession=" GDMCONFDIR "/Xsession"
 #define GDM_KEY_DEFAULT_SESSION "daemon/DefaultSession=gnome.desktop"
 #define GDM_KEY_SUSPEND "daemon/SuspendCommand=" SUSPEND_COMMAND
 
@@ -243,6 +268,9 @@ enum {
 
 #define GDM_KEY_CONSOLE_CANNOT_HANDLE "daemon/ConsoleCannotHandle=am,ar,az,bn,el,fa,gu,hi,ja,ko,ml,mr,pa,ta,zh"
 
+/* How long to wait before assuming an Xserver has timed out */
+#define GDM_KEY_XSERVER_TIMEOUT "daemon/GdmXserverTimeout=10"
+
 /* Per server definitions */
 #define GDM_KEY_SERVER_PREFIX "server-"
 #define GDM_KEY_SERVER_NAME "name=Standard server"
@@ -266,6 +294,7 @@ enum {
 #define GDM_KEY_CHECK_DIR_OWNER "security/CheckDirOwner=true"
 #define GDM_KEY_RETRY_DELAY "security/RetryDelay=1"
 #define GDM_KEY_DISALLOW_TCP "security/DisallowTCP=true"
+#define GDM_KEY_PAM_STACK "security/PamStack=gdm"
 
 #define GDM_KEY_NEVER_PLACE_COOKIES_ON_NFS "security/NeverPlaceCookiesOnNFS=true"
 #define GDM_KEY_PASSWORD_REQUIRED "security/PasswordRequired=false"
@@ -280,14 +309,14 @@ enum {
 #define GDM_KEY_MAX_INDIRECT "xdmcp/MaxPendingIndirect=4"
 #define GDM_KEY_MAX_WAIT_INDIRECT "xdmcp/MaxWaitIndirect=15"
 #define GDM_KEY_PING_INTERVAL "xdmcp/PingIntervalSeconds=15"
-#define GDM_KEY_WILLING "xdmcp/Willing=" EXPANDED_SYSCONFDIR "/gdm/Xwilling"
+#define GDM_KEY_WILLING "xdmcp/Willing=" GDMCONFDIR "/Xwilling"
 
 #define GDM_KEY_XDMCP_PROXY "xdmcp/EnableProxy=false"
 #define GDM_KEY_XDMCP_PROXY_XSERVER "xdmcp/ProxyXServer="
 #define GDM_KEY_XDMCP_PROXY_RECONNECT "xdmcp/ProxyReconnect="
 
 #define GDM_KEY_GTK_THEME "gui/GtkTheme=Default"
-#define GDM_KEY_GTKRC "gui/GtkRC=" EXPANDED_DATADIR "/themes/Default/gtk-2.0/gtkrc"
+#define GDM_KEY_GTKRC "gui/GtkRC=" DATADIR "/themes/Default/gtk-2.0/gtkrc"
 #define GDM_KEY_MAX_ICON_WIDTH "gui/MaxIconWidth=128"
 #define GDM_KEY_MAX_ICON_HEIGHT "gui/MaxIconHeight=128"
 
@@ -299,14 +328,14 @@ enum {
 #define GDM_KEY_EXCLUDE "greeter/Exclude=bin,daemon,adm,lp,sync,shutdown,halt,mail,news,uucp,operator,nobody,gdm,postgres,pvm,rpm,nfsnobody,pcap"
 #define GDM_KEY_INCLUDE_ALL "greeter/IncludeAll=false"
 #define GDM_KEY_MINIMAL_UID "greeter/MinimalUID=100"
-#define GDM_KEY_DEFAULT_FACE "greeter/DefaultFace=" EXPANDED_PIXMAPDIR "/nobody.png"
-#define GDM_KEY_GLOBAL_FACE_DIR "greeter/GlobalFaceDir=" EXPANDED_DATADIR "/pixmaps/faces/"
-#define GDM_KEY_LOCALE_FILE "greeter/LocaleFile=" EXPANDED_LOCALEDIR "/locale.alias"
-#define GDM_KEY_LOGO "greeter/Logo=" EXPANDED_PIXMAPDIR "/gdm-foot-logo.png"
-#define GDM_KEY_CHOOSER_BUTTON_LOGO "greeter/ChooserButtonLogo=" EXPANDED_PIXMAPDIR "/gdm-foot-logo.png"
+#define GDM_KEY_DEFAULT_FACE "greeter/DefaultFace=" PIXMAPDIR "/nobody.png"
+#define GDM_KEY_GLOBAL_FACE_DIR "greeter/GlobalFaceDir=" DATADIR "/pixmaps/faces/"
+#define GDM_KEY_LOCALE_FILE "greeter/LocaleFile=" GDMLOCALEDIR "/locale.alias"
+#define GDM_KEY_LOGO "greeter/Logo=" PIXMAPDIR "/gdm-foot-logo.png"
+#define GDM_KEY_CHOOSER_BUTTON_LOGO "greeter/ChooserButtonLogo=" PIXMAPDIR "/gdm-foot-logo.png"
 #define GDM_KEY_QUIVER "greeter/Quiver=true"
 #define GDM_KEY_SYSTEM_MENU "greeter/SystemMenu=true"
-#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" EXPANDED_SBINDIR "/gdmsetup --disable-sound --disable-crash-dialog"
+#define GDM_KEY_CONFIGURATOR "daemon/Configurator=" SBINDIR "/gdmsetup --disable-sound --disable-crash-dialog"
 #define GDM_KEY_CONFIG_AVAILABLE "greeter/ConfigAvailable=true"
 #define GDM_KEY_CHOOSER_BUTTON "greeter/ChooserButton=true"
 #define GDM_KEY_TITLE_BAR "greeter/TitleBar=true"
@@ -345,19 +374,19 @@ enum {
 #define GDM_KEY_SET_POSITION "greeter/SetPosition=false"
 #define GDM_KEY_POSITION_X "greeter/PositionX=0"
 #define GDM_KEY_POSITION_Y "greeter/PositionY=0"
-#define GDM_KEY_USE_24_CLOCK "greeter/Use24Clock=false"
+#define GDM_KEY_USE_24_CLOCK "greeter/Use24Clock=auto"
 #define GDM_KEY_ENTRY_CIRCLES "greeter/UseCirclesInEntry=false"
 #define GDM_KEY_ENTRY_INVISIBLE "greeter/UseInvisibleInEntry=false"
 #define GDM_KEY_GRAPHICAL_THEME "greeter/GraphicalTheme=circles"
 #define GDM_KEY_GRAPHICAL_THEMES "greeter/GraphicalThemes=circles"
 #define GDM_KEY_GRAPHICAL_THEME_RAND "greeter/GraphicalThemeRand=false"
-#define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" EXPANDED_DATADIR "/gdm/themes/"
-#define GDM_KEY_GRAPHICAL_THEME_COLOR "greeter/GraphicalThemedColor=#76848F"
+#define GDM_KEY_GRAPHICAL_THEME_DIR "greeter/GraphicalThemeDir=" DATADIR "/gdm/themes/"
+#define GDM_KEY_GRAPHICAL_THEMED_COLOR "greeter/GraphicalThemedColor=#76848F"
 
 #define GDM_KEY_INFO_MSG_FILE "greeter/InfoMsgFile="
 #define GDM_KEY_INFO_MSG_FONT "greeter/InfoMsgFont="
 
-#define GDM_KEY_POST_DISPLAY_PROGRAM "greeter/PostDisplayProgram="
+#define GDM_KEY_PRE_FETCH_PROGRAM "greeter/PreFetchProgram="
 
 #define GDM_KEY_SOUND_ON_LOGIN "greeter/SoundOnLogin=true"
 #define GDM_KEY_SOUND_ON_LOGIN_SUCCESS "greeter/SoundOnLoginSuccess=false"
@@ -368,8 +397,8 @@ enum {
 #define GDM_KEY_SOUND_PROGRAM "daemon/SoundProgram=" SOUND_PROGRAM
 
 #define GDM_KEY_SCAN_TIME "chooser/ScanTime=4"
-#define GDM_KEY_DEFAULT_HOST_IMG "chooser/DefaultHostImg=" EXPANDED_PIXMAPDIR "/nohost.png"
-#define GDM_KEY_HOST_IMAGE_DIR "chooser/HostImageDir=" EXPANDED_DATADIR "/hosts/"
+#define GDM_KEY_DEFAULT_HOST_IMG "chooser/DefaultHostImg=" PIXMAPDIR "/nohost.png"
+#define GDM_KEY_HOST_IMAGE_DIR "chooser/HostImageDir=" DATADIR "/hosts/"
 #define GDM_KEY_HOSTS "chooser/Hosts="
 #define GDM_KEY_MULTICAST "chooser/Multicast=true"
 #define GDM_KEY_MULTICAST_ADDR "chooser/MulticastAddr=ff02::1"
@@ -716,10 +745,12 @@ void		gdm_final_cleanup	(void);
 #define GDM_SLAVE_NOTIFY_COMMAND '#'
 
 /*
- * Maximum number of connections allowed at once.  Might be useful to
- * try and tune this.  GDM would be faster if the value is larger.
+ * Maximum number of messages allowed over the sockets protocol.  This
+ * is set to 80 since the gdmlogin/gdmgreeter programs have ~60 config
+ * values that are pulled over the socket connection so it allows them
+ * all to be grabbed in one pull.
  */
-#define GDM_SUP_MAX_CONNECTIONS 20
+#define GDM_SUP_MAX_MESSAGES 80
 #define GDM_SUP_SOCKET "/tmp/.gdm_socket"
 
 /*
@@ -747,8 +778,8 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_AUTH_LOCAL "AUTH_LOCAL" /* <xauth cookie> */
-/* AUTH_LOCAL: Setup this connection as authenticated for FLEXI_SERVER
- *             Because all full blown (non-Xnest) servers can be started
+/* AUTH_LOCAL: Setup this connection as authenticated for FLEXI_SERVER.
+ *             Because all full blown (non-Xnest) displays can be started
  *             only from users logged in locally, and here gdm assumes
  *             only users logged in from gdm.  They must pass the xauth
  *             MIT-MAGIC-COOKIE-1 that they were passed before the
@@ -772,11 +803,11 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XSERVER "FLEXI_XSERVER" /* <xserver type> */
-/* FLEXI_XSERVER: Start a new X flexible server.  Only supported on
+/* FLEXI_XSERVER: Start a new X flexible display.  Only supported on
  *                connection that passed AUTH_LOCAL
  * Supported since: 2.2.4.0
  * Arguments: <xserver type>
- *   If no arguments, starts the standard x server
+ *   If no arguments, starts the standard X server
  * Answers:
  *   OK <display>
  *   ERROR <err number> <english error description>
@@ -791,7 +822,7 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <uid> <xauth cookie> <xauth file> */
-/* FLEXI_XNEXT: Start a new flexible Xnest server.
+/* FLEXI_XNEXT: Start a new flexible Xnest display.
  * Note:        Supported on older versions from 2.2.4.0, later
  *              2.2.4.2, but since 2.3.90.4 you must supply 4 
  *              arguments or ERROR 100 will be returned.  This
@@ -847,14 +878,15 @@ void		gdm_final_cleanup	(void);
  */
 #define GDM_SUP_RELEASE_DYNAMIC_DISPLAYS	"RELEASE_DYNAMIC_DISPLAYS"
 /*
- * RELEASE_DYNAMIC_DISPLAYS: Release dynamic displays currently in
- *                           DISPLAY_CONFIG state
+ * RELEASE_DYNAMIC_DISPLAYS: Release and begin managing dynamic displays
+ *                           currently in DISPLAY_CONFIG state
  * Supported since: 2.8.0.0
- * Arguments: None
+ * Arguments: <display to release>
  * Answers:
  *  OK
  *  ERROR
  *     0 = Not implemented
+ *     1 = Bad display number
  *     100 = Not authenticated
  *     200 = Dynamic Displays not allowed
  *     999 = Unknown error
@@ -869,14 +901,15 @@ void		gdm_final_cleanup	(void);
  *  OK
  *  ERROR
  *     0 = Not implemented
+ *     1 = Bad display number
  *     100 = Not authenticated
  *     200 = Dynamic Displays not allowed
  *     999 = Unknown error
  */
 #define GDM_SUP_ATTACHED_SERVERS "ATTACHED_SERVERS" /* None */
 #define GDM_SUP_CONSOLE_SERVERS  "CONSOLE_SERVERS"  /* None */
-/* ATTACHED_SERVERS: List all attached servers.  Doesn't list XDMCP
- *                   and xnest non-attached servers
+/* ATTACHED_SERVERS: List all attached displays.  Doesn't list XDMCP
+ *                   and xnest non-attached displays.
  * Note:             This command used to be named CONSOLE_SERVERS,
  *                   which is still recognized for backwards
  *                   compatibility.  The optional pattern argument
@@ -905,7 +938,7 @@ void		gdm_final_cleanup	(void);
 #define GDM_SUP_ALL_SERVERS  "ALL_SERVERS" /* None */
 /* ALL_SERVERS: List all displays, including attached, remote, xnest.
  *              This can for example be useful to figure out if
- *              the server you are on is managed by the gdm daemon,
+ *              the display you are on is managed by the gdm daemon,
  *              by seeing if it is in the list.  It is also somewhat
  *              like the 'w' command but for graphical sessions.
  * Supported since: 2.4.2.96
@@ -963,14 +996,18 @@ void		gdm_final_cleanup	(void);
 /* GET_CONFIG:  Get configuration value for key.  Useful so
  *              that other applications can request configuration
  *              information from GDM.  Any key defined as GDM_KEY_*
- *              in gdm.h is * supported.  Starting with version 2.13.0.2
+ *              in gdm.h is supported.  Starting with version 2.13.0.2
  *              translated keys (such as "greeter/GdmWelcome[cs]" are
  *              supported via GET_CONFIG.  Also starting with version
  *              2.13.0.2 it is no longer necessary to include the
  *              default value (i.e. you can use key "greeter/IncludeAll"
  *              instead of having to use "greeter/IncludeAll=false".
+ *              Starting with version 2.14.4, GDM supports per-display
+ *              configuration (for GUI slave clients only) and accepts
+ *              the display argument to retrieve per-display value (if
+ *              any.
  * Supported since: 2.6.0.9
- * Arguments: <key>
+ * Arguments: <key> [<display>]
  * Answers:
  *   OK <value>
  *   ERROR <err number> <english error description>
@@ -980,7 +1017,7 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_GET_CONFIG_FILE  "GET_CONFIG_FILE" /* None */
-/* GET_CONFIG_FILE: Get config file location being used by
+/* GET_CONFIG_FILE: Get defaults config file location being used by
  #                  the daemon.   If the GDM daemon was started
  *                  with the --config option, it will return
  *                  the value passed in via that argument.
@@ -993,6 +1030,19 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_GET_CUSTOM_CONFIG_FILE  "GET_CUSTOM_CONFIG_FILE" /* None */
+/* GET_CONFIG_FILE: Get custom config file location being used by
+ #                  the daemon. 
+ * Supported since: 2.14.0.0
+ * Arguments: None
+ * Answers:
+ *   OK <full path to GDM custom configuration file>
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      1 = File not found
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
 #define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
 /* UPDATE_CONFIG: Tell the daemon to re-read a key from the
  *                GDM configuration file.   Any user can request
@@ -1097,7 +1147,7 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_SET_LOGOUT_ACTION "SET_LOGOUT_ACTION" /* <action> */
-/* SET_LOGOUT_ACTION: Tell the daemon to halt/reboot/suspend after
+/* SET_LOGOUT_ACTION: Tell the daemon to halt/restart/suspend after
  *                    slave process exits.  Only supported on
  *                    connections that passed AUTH_LOCAL.
  * Supported since: 2.5.90.0
@@ -1116,7 +1166,7 @@ void		gdm_final_cleanup	(void);
  *      999 = Unknown error
  */
 #define GDM_SUP_SET_SAFE_LOGOUT_ACTION "SET_SAFE_LOGOUT_ACTION" /* <action> */
-/* SET_SAFE_LOGOUT_ACTION: Tell the daemon to halt/reboot/suspend
+/* SET_SAFE_LOGOUT_ACTION: Tell the daemon to halt/restart/suspend
  *                         after everybody logs out.  If only one
  *                         person logs out, then this is obviously
  *                         the same as the SET_LOGOUT_ACTION.  Note
@@ -1186,6 +1236,20 @@ void		gdm_final_cleanup	(void);
  *      200 = Too many messages
  *      999 = Unknown error
  */
+#define GDM_SUP_SERVER_BUSY "SERVER_BUSY" /* None */
+/* SERVER_BUSY:  Returns true if half or more of the daemon's sockets are
+ *               busy, false otherwise.  Used by slave programs which want
+ *               to ensure they do not overhwelm the server.
+ * Supported since: 2.13.0.8
+ * Arguments: None 
+ * Answers:
+ *   OK <value>
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      200 = Too many messages
+ *      999 = Unknown error
+ */
+#define GDM_SUP_GET_SERVER_DETAILS "GET_SERVER_DETAILS" /* <server> <key> */
 #define GDM_SUP_CLOSE        "CLOSE" /* None */
 /* CLOSE: Close sockets connection
  * Supported since: 2.2.4.0
