@@ -25,6 +25,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <popt.h>
 #include <sys/types.h>
 #include <signal.h>
 #include <sys/socket.h>
@@ -36,27 +37,25 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h> /* for gnome_config */
-#include <libgnomeui/libgnomeui.h> /* for gnome_program */
-
-#include <viciousui.h>
 
 #include "gdm.h"
 #include "gdmcomm.h"
 #include "gdmcommon.h"
+#include "gdmconfig.h"
+#include "gdmconfig.h"
 
-static GSList *xservers = NULL;
-static gboolean got_standard = FALSE;
-static gboolean use_xnest = FALSE;
-static gboolean authenticate = FALSE;
-static gboolean no_lock = FALSE;
-static gboolean monte_carlo_pi = FALSE;
-static const char *send_command = NULL;
-static const char *server = NULL;
+static GSList *xservers          = NULL;
+static const char *send_command  = NULL;
+static const char *server        = NULL;
 static const char *chosen_server = NULL;
-static gboolean debug = FALSE;
-static gboolean startnew = FALSE;
-static char *auth_cookie = NULL;
+static char *auth_cookie         = NULL;
+static gboolean got_standard     = FALSE;
+static gboolean use_xnest        = FALSE;
+static gboolean debug_in         = FALSE;
+static gboolean authenticate     = FALSE;
+static gboolean no_lock          = FALSE;
+static gboolean monte_carlo_pi   = FALSE;
+static gboolean startnew         = FALSE;
 
 static int
 get_cur_vt (void)
@@ -101,9 +100,8 @@ change_vt (int vt)
 			 GTK_DIALOG_MODAL /* flags */,
 			 GTK_MESSAGE_ERROR,
 			 GTK_BUTTONS_OK,
-			 FALSE /* markup */,
 			 _("Cannot change display"),
-			 "%s", message);
+			 message);
 
 		gtk_widget_show_all (dialog);
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -334,7 +332,7 @@ maybe_lock_screen (void)
 	screen = gdk_screen_get_default ();
 
 	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
-		g_warning (_("Can't lock screen"));
+		g_warning ("Cannot lock screen: %s", error->message);
 		g_error_free (error);
 	}
 
@@ -347,7 +345,7 @@ maybe_lock_screen (void)
 	}
 
 	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
-		g_warning (_("Can't disable xscreensaver display hacks"));
+		g_warning ("Cannot disable screensaver engines: %s", error->message);
 		g_error_free (error);
 	}
 
@@ -365,11 +363,12 @@ run_logged_in_dialogue (char **vec)
 	GtkWidget *treeview;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
+	gint response;
 
-        if (startnew == TRUE) {
-                /* Just return if the user doesn't want to see the dialog */
-                return;
-	}
+	if (startnew == TRUE) {
+		/* Just return if the user doesn't want to see the dialog */
+		return;
+	} 
 
 	dialog = gtk_dialog_new_with_buttons (_("Open Displays"),
 					      NULL /* parent */,
@@ -385,7 +384,7 @@ run_logged_in_dialogue (char **vec)
 	vbox = GTK_DIALOG (dialog)->vbox;
 
 	w = gtk_label_new (_("There are some displays already open.  You can select "
-			     "one from the list below or open a new one."));
+		     "one from the list below or open a new one."));
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -419,20 +418,22 @@ run_logged_in_dialogue (char **vec)
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 
-        g_signal_connect (selection, "changed",
+	g_signal_connect (selection, "changed",
 			  G_CALLBACK (selection_changed),
 			  dialog);
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   RESPONSE_OPEN_EXISTING_DISPLAY,
-					   FALSE);
+				   RESPONSE_OPEN_EXISTING_DISPLAY,
+				   FALSE);
 
 	gtk_widget_show_all (dialog);
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 run_again:
-	switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
+	switch (response) {
 	case RESPONSE_OPEN_NEW_DISPLAY:
 		gtk_widget_destroy (dialog);
+
 		/* just continue what you are doing */
 		return;
 
@@ -454,8 +455,8 @@ run_again:
 
 			change_vt (vt);
 
-			/* FIXME: wait + disturb the pointer (need SUP?), perhaps
-			 * part of the sup command to CHVT ?? */
+			/* FIXME: wait + disturb the pointer (need SUP?), 
+			 * perhaps part of the sup command to CHVT ?? */
 
 			exit (0);
 		} else {
@@ -512,7 +513,7 @@ check_for_users (void)
 		if (strcmp (rvec[0], gdmcomm_get_display ()) != 0 &&
 		    vt >= 0) {
 			/* this is not the current display */
-			extra ++;
+			extra++;
 		}
 
 		g_strfreev (rvec);
@@ -529,62 +530,21 @@ check_for_users (void)
 }
 
 static void
-read_servers (gchar *config_file)
+read_servers (void)
 {
-	gpointer iter;
-	gchar *config_sections;
-	char *k;
+	GSList *li;
 
-	/* Find server definitions */
-	config_sections = g_strdup_printf ("=%s=/", config_file);
-	iter = gnome_config_init_iterator_sections (config_sections);
-	iter = gnome_config_iterator_next (iter, &k, NULL);
+        xservers = gdm_config_get_xservers (TRUE);
 
-	while (iter) {
-		if (strncmp (k, "server-", strlen ("server-")) == 0) {
-			char *section;
-			GdmXServer *svr;
+	for (li = xservers; li != NULL; li = li->next) {
+		GdmXserver *svr = li->data;
 
-			section = g_strdup_printf ("=%s=/%s/", config_file, k);
-			gnome_config_push_prefix (section);
+		if (strcmp (svr->id, GDM_STANDARD) == 0)
+			got_standard = TRUE;
 
-			if ( ! gnome_config_get_bool
-			     (GDM_KEY_SERVER_FLEXIBLE)) {
-				gnome_config_pop_prefix ();
-				g_free (section);
-				g_free (k);
-				iter = gnome_config_iterator_next (iter,
-								   &k, NULL);
-				continue;
-			}
-
-			svr = g_new0 (GdmXServer, 1);
-
-			svr->id = g_strdup (k + strlen ("server-"));
-			svr->name = gnome_config_get_string
-				(GDM_KEY_SERVER_NAME);
-			svr->command = gnome_config_get_string
-				(GDM_KEY_SERVER_COMMAND);
-			svr->flexible = TRUE;
-			svr->choosable = gnome_config_get_bool
-				(GDM_KEY_SERVER_CHOOSABLE);
-
-			g_free (section);
-			gnome_config_pop_prefix ();
-
-			if (strcmp (svr->id, GDM_STANDARD) == 0)
-				got_standard = TRUE;
-
-			if (server != NULL &&
-			    strcmp (svr->id, server) == 0)
-				chosen_server = g_strdup (svr->id);
-
-			xservers = g_slist_append (xservers, svr);
-		}
-
-		g_free (k);
-
-		iter = gnome_config_iterator_next (iter, &k, NULL);
+		if (server != NULL &&
+		    strcmp (svr->id, server) == 0)
+			chosen_server = g_strdup (svr->id);
 	}
 }
 
@@ -630,7 +590,7 @@ choose_server (void)
 	}
 
 	for (li = xservers; li != NULL; li = li->next) {
-		GdmXServer *svr = li->data;
+		GdmXserver *svr = li->data;
 		w = gtk_radio_button_new_with_label
 			(group, svr->name ? svr->name : svr->id);
 		gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
@@ -670,6 +630,42 @@ choose_server (void)
 	return NULL;
 }
 
+/**
+ * is_key
+ *
+ * Since GDM keys sometimes have default values defined in the gdm.h header
+ * file (e.g. key=value), this function strips off the "=value" from both
+ * keys passed in to do a comparison.
+ */
+static gboolean
+is_key (const gchar *key1, const gchar *key2)
+{
+   gchar *key1d, *key2d, *p;
+
+   key1d = g_strdup (key1);
+   key2d = g_strdup (key2);
+
+   g_strstrip (key1d);
+   p = strchr (key1d, '=');
+   if (p != NULL)
+      *p = '\0';
+
+   g_strstrip (key2d);
+   p = strchr (key2d, '=');
+   if (p != NULL)
+      *p = '\0';
+
+   if (strcmp (key1d, key2d) == 0) {
+      g_free (key1d);
+      g_free (key2d);
+      return TRUE;
+   } else {
+      g_free (key1d);
+      g_free (key2d);
+      return FALSE;
+   }
+}
+
 static void
 calc_pi (void)
 {
@@ -689,12 +685,12 @@ calc_pi (void)
 }
 
 struct poptOption options [] = {
-	{ "command", 'c', POPT_ARG_STRING, &send_command, 0, N_("Send the specified protocol command to gdm"), N_("COMMAND") },
+	{ "command", 'c', POPT_ARG_STRING, &send_command, 0, N_("Send the specified protocol command to GDM"), N_("COMMAND") },
 	{ "xnest", 'n', POPT_ARG_NONE, &use_xnest, 0, N_("Xnest mode"), NULL },
 	{ "no-lock", 'l', POPT_ARG_NONE, &no_lock, 0, N_("Do not lock current screen"), NULL },
-	{ "debug", 'd', POPT_ARG_NONE, &debug, 0, N_("Debugging output"), NULL },
+	{ "debug", 'd', POPT_ARG_NONE, &debug_in, 0, N_("Debugging output"), NULL },
 	{ "authenticate", 'a', POPT_ARG_NONE, &authenticate, 0, N_("Authenticate before running --command"), NULL },
-        { "startnew", 's', POPT_ARG_NONE, &startnew, 0, N_("Start new flexible session; do not show popup"), NULL },
+	{ "startnew", 's', POPT_ARG_NONE, &startnew, 0, N_("Start new flexible session; do not show popup"), NULL },
 	{ "monte-carlo-pi", 0, POPT_ARG_NONE, &monte_carlo_pi, 0, NULL, NULL },
 	POPT_AUTOHELP
 	{ NULL, 0, 0, NULL, 0}
@@ -705,58 +701,100 @@ int
 main (int argc, char *argv[])
 {
 	GtkWidget *dialog;
-	gchar *config_file;
 	char *command;
 	char *version;
 	char *ret;
 	const char *message;
 	poptContext ctx;
 	const char **args;
-	GnomeProgram *program;
+	int nextopt;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	program = gnome_program_init ("gdmflexiserver", VERSION, 
-				      LIBGNOMEUI_MODULE /* module_info */,
-				      argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
-				      NULL);
-	g_object_get (G_OBJECT (program),
-		      GNOME_PARAM_POPT_CONTEXT, &ctx,
-		      NULL);	
+	/* Option parsing */
+
+	ctx = poptGetContext(NULL, argc, (const char**)argv, options, 0);
+	while ((nextopt = poptGetNextOpt(ctx)) > 0 || nextopt == POPT_ERROR_BADOPT);
 
 	if (monte_carlo_pi) {
 		calc_pi ();
 		return 0;
 	}
 
-	gdmcomm_set_debug (debug);
+	gdmcomm_set_debug (debug_in);
 
 	args = poptGetArgs (ctx);
 	if (args != NULL && args[0] != NULL)
 		server = args[0];
 
-	config_file = gdm_common_get_config_file ();
-	if (config_file == NULL) {
-		g_print (_("Could not access GDM configuration file.\n"));
-		exit (0);
+	if (send_command != NULL) {
+		if ( ! gdmcomm_check (FALSE)) {
+			gdm_common_error ("Error: GDM (GNOME Display Manager) "
+			"is not running.");
+			gdm_common_error ("You might be using a different "
+			"display manager.");
+			return 1;
+		}
+	} else {
+		/*
+		 * The --command argument does not display anything, so avoid
+		 * running gtk_init until it finishes.  Sometimes the
+		 * --command argument is used when there is no display so it
+		 * will fail and cause the program to exit, complaining about
+		 * "no display".
+		 */
+		gtk_init(&argc, &argv);
+		if ( ! gdmcomm_check (TRUE)) {
+			return 1;
+		}
 	}
 
-	if ( ! gdmcomm_check (config_file, TRUE /* gui_bitching */)) {
-		g_free (config_file);
-		return 1;
-	}
+	/* Start reading config data in bulk */
+	gdmcomm_comm_bulk_start ();
+
+	/* Process --command option */
 
 	if (send_command != NULL) {
 		if (authenticate)
 			auth_cookie = gdmcomm_get_auth_cookie ();
-		ret = gdmcomm_call_gdm (send_command, auth_cookie,
-					"2.2.4.0", 5);
+
+		/*
+		 * If asking for a translatable config value, then try to get
+		 * the translated value first.  If this fails, then go ahead
+		 * and call the normal sockets command.
+		 */
+		if (strncmp (send_command, GDM_SUP_GET_CONFIG " ",
+		    strlen (GDM_SUP_GET_CONFIG " ")) == 0) {
+			gchar *value = NULL;
+			const char *key = &send_command[strlen (GDM_SUP_GET_CONFIG " ")];
+
+			if (is_key (GDM_KEY_WELCOME, key) ||
+			    is_key (GDM_KEY_REMOTE_WELCOME, key)) {
+				value = gdm_config_get_translated_string ((gchar *)key);
+				if (value != NULL) {
+					ret = g_strdup_printf ("OK %s", value);
+				}
+			}
+
+			/*
+			 * If the above didn't return a value, then must be a
+			 * different key, so call gdmcomm_call_gdm.
+			 */
+			if (value == NULL)
+				ret = gdmcomm_call_gdm (send_command, auth_cookie,
+							"2.2.4.0", 5);
+		} else {
+			ret = gdmcomm_call_gdm (send_command, auth_cookie,
+						"2.2.4.0", 5);
+		}
+
+		/* At this point we are done using the socket, so close it */
+		gdmcomm_comm_bulk_stop ();
+
 		if (ret != NULL) {
 			g_print ("%s\n", ret);
-			g_free (config_file);
 			return 0;
 		} else {
 			dialog = ve_hig_dialog_new
@@ -764,22 +802,34 @@ main (int argc, char *argv[])
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("Cannot communicate with GDM "
 				   "(The GNOME Display Manager)"),
-				 "%s",
 				 _("Perhaps you have an old version "
 				   "of GDM running."));
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-			g_free (config_file);
 			return 1;
 		}
 	}
 
-	/* always attempt to get cookie and authenticate.  On remote
-	servers */
+	/*
+	 * Now process what gdmflexiserver is more frequently used to
+	 * do, start VT (Virtual Terminal) sesions - at least on
+	 * systems where it is supported.  On systems where it is not
+	 * supporteed VT stands for "Very Tight" and will mess up your
+	 * display if you use it.  Tight!  So do not use it.
+	 *
+	 * I would accept a patch to disable it on such systems, but it
+	 * is easy to avoid not using it as long as your distro does not
+	 * put the menu choice in the application launch button on the
+	 * panel (don't ship the desktop file).
+	 */
+
+	/*
+	 * Always attempt to get cookie and authenticate.  On remote
+	 * servers
+	 */
 	auth_cookie = gdmcomm_get_auth_cookie ();
 
 	/* check for other displays/logged in users */
@@ -787,24 +837,26 @@ main (int argc, char *argv[])
 
 	if (use_xnest) {
 		char *cookie = gdmcomm_get_a_cookie (FALSE /* binary */);
+
 		if (cookie == NULL) {
+
+			/* At this point we are done using the socket, so close it */
+			gdmcomm_comm_bulk_stop ();
+
 			dialog = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("You do not seem to have the "
 				   "authentication needed for this "
 				   "operation"),
-				 "%s",
 				 _("Perhaps your .Xauthority "
 				   "file is not set up correctly."));
 
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-			g_free (config_file);
 			return 1;
 		}
 		command = g_strdup_printf (GDM_SUP_FLEXI_XNEST " %s %d %s %s",
@@ -817,15 +869,17 @@ main (int argc, char *argv[])
 		auth_cookie = NULL;
 	} else {
 		if (auth_cookie == NULL) {
+
+			/* At this point we are done using the socket, so close it */
+			gdmcomm_comm_bulk_stop ();
+
 			dialog = ve_hig_dialog_new
 				(NULL /* parent */,
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("You do not seem to be logged in on the "
 				   "console"),
-				 "%s",
 				 _("Starting a new login only "
 				   "works correctly on the console."));
 			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
@@ -833,11 +887,10 @@ main (int argc, char *argv[])
 			gtk_widget_show_all (dialog);
 			gtk_dialog_run (GTK_DIALOG (dialog));
 			gtk_widget_destroy (dialog);
-			g_free (config_file);
 			return 1;
 		}
 
-		read_servers (config_file);
+		read_servers ();
 		server = choose_server ();
 		if (server == NULL)
 			command = g_strdup (GDM_SUP_FLEXI_XSERVER);
@@ -847,18 +900,19 @@ main (int argc, char *argv[])
 		version = "2.2.4.0";
 	}
 
-	g_free (config_file);
-
 	ret = gdmcomm_call_gdm (command, auth_cookie, version, 5);
 	g_free (command);
 
+	/* At this point we are done using the socket, so close it */
+	gdmcomm_comm_bulk_stop ();
+
 	if (ret != NULL &&
 	    strncmp (ret, "OK ", 3) == 0) {
 
 		/* if we switched to a different screen as a result of this,
 		 * lock the current screen */
 		if ( ! no_lock && ! use_xnest) {
-				maybe_lock_screen ();
+			maybe_lock_screen ();
 		}
 
 		/* all fine and dandy */
@@ -872,9 +926,8 @@ main (int argc, char *argv[])
 		 GTK_DIALOG_MODAL /* flags */,
 		 GTK_MESSAGE_ERROR,
 		 GTK_BUTTONS_OK,
-		 FALSE /* markup */,
 		 _("Cannot start new display"),
-		 "%s", message);
+		 message);
 
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
@@ -915,7 +968,7 @@ torture (void)
 		int len = rand () % 5000;
 		char *buf = g_new (char, len);
 		int ii;
-		for (ii = 0; ii < len; ii ++)
+		for (ii = 0; ii < len; ii++)
 			buf[ii] = rand () % 256;
 		write (fd, buf, len); 
 		g_free (buf);
@@ -932,7 +985,7 @@ torture_test (void)
 
 	srand (getpid () * time (NULL));
 
-	for (i = 0; i < 500; i ++) {
+	for (i = 0; i < 500; i++) {
 		if (fork () == 0) {
 			torture ();
 			_exit (0);
