@@ -114,37 +114,6 @@ parse_id (xmlNodePtr node,
     }
 }
 
-static gboolean
-parse_button (xmlNodePtr node,
-	      GreeterItemInfo *info,
-	      GError **error)
-{
-  xmlChar *prop;
-  
-  prop = xmlGetProp (node,(const xmlChar *) "button");
-  if (prop)
-    {
-      if (strcmp ((char *) prop, "true") == 0)
-	{
-	  info->button = TRUE;
-	}
-      else if (strcmp ((char *) prop, "false") == 0)
-	{
-	  info->button = FALSE;
-	}
-      else
-	{
-	  g_set_error (error,
-		       GREETER_PARSER_ERROR,
-		       GREETER_PARSER_ERROR_BAD_SPEC,
-		       "bad button spec %s", prop);
-	  xmlFree (prop);
-	  return FALSE;
-	}
-      xmlFree (prop);
-    }
-  return TRUE;
-}
 
 /* Doesn't set the parts of rect that are not specified.
  * If you want specific default values you need to fill them out
@@ -1129,7 +1098,12 @@ parse_stock (xmlNodePtr node,
 	  g_free (*translated_text);
 	  *translated_text = g_strdup (_("_OK"));
 	}
-      else if (g_ascii_strcasecmp ((char *) prop, "cancel") == 0)
+      /*
+       * Support startagain as cancel for forward compatibility without
+       * string breakage.
+       */
+      else if (g_ascii_strcasecmp ((char *) prop, "cancel") == 0 ||
+               g_ascii_strcasecmp ((char *) prop, "startagain") == 0)
         {
 	  g_free (*translated_text);
 	  *translated_text = g_strdup (_("_Cancel"));
@@ -1186,6 +1160,88 @@ do_font_size_reduction (GreeterItemInfo *info)
     }
 }
 
+static gboolean
+parse_canvasbutton (xmlNodePtr node,
+		    GreeterItemInfo *info,
+		    GError **error)
+{
+  xmlChar *prop;
+  
+  prop = xmlGetProp (node,(const xmlChar *) "button");
+  if (prop)
+    {
+      if (strcmp ((char *) prop, "true") == 0)
+	{
+	  info->canvasbutton = TRUE;
+	}
+      else if (strcmp ((char *) prop, "false") == 0)
+	{
+	  info->canvasbutton = FALSE;
+	}
+      else
+	{
+	  g_set_error (error,
+		       GREETER_PARSER_ERROR,
+		       GREETER_PARSER_ERROR_BAD_SPEC,
+		       "bad button spec %s", prop);
+	  xmlFree (prop);
+	  return FALSE;
+	}
+      xmlFree (prop);
+    }
+  return TRUE;
+}
+
+static gboolean
+parse_gtkbutton (xmlNodePtr node,
+		    GreeterItemInfo *info,
+		    GError **error)
+{
+  xmlNodePtr child;
+  char *translated_text  = NULL;
+  gint translation_score = 1000;
+
+  child = node->children;
+
+  while (child)
+    {
+      if (strcmp ((char *) child->name, "pos") == 0)
+	{
+	  if G_UNLIKELY (!parse_pos (child, info, error))
+	    return FALSE;
+	}
+      else if (child->type == XML_ELEMENT_NODE &&
+	       strcmp ((char *) child->name, "stock") == 0)
+	{
+	  if G_UNLIKELY (!parse_stock (child, info, &translated_text, &translation_score, error))
+	    return FALSE;
+	}
+
+      child = child->next;
+    }
+
+  if (translated_text == NULL)
+    {
+      g_set_error (error,
+                   GREETER_PARSER_ERROR,
+                   GREETER_PARSER_ERROR_BAD_SPEC,
+                   "A label must specify the text attribute");
+      return FALSE;
+    }
+
+  /* FIXME: evil hack to use internally translated strings */
+  if (translation_score == 999 &&
+      ! ve_string_empty (translated_text))
+    {
+      char *foo = g_strdup (_(translated_text));
+      g_free (translated_text);
+      translated_text = foo;
+    }
+
+  info->data.text.orig_text = translated_text;
+
+  return TRUE;
+}
 
 static gboolean
 parse_label_pos_extras (xmlNodePtr       node,
@@ -1240,11 +1296,9 @@ parse_label (xmlNodePtr        node,
 {
   xmlNodePtr child;
   int i;
-  char *translated_text;
+  char *translated_text  = NULL;
   gint translation_score = 1000;
   
-  translated_text = NULL;
-
   child = node->children;
   while (child)
     {
@@ -1341,10 +1395,10 @@ parse_listitem (xmlNodePtr        node,
 		GError         **error)
 {
   xmlNodePtr child;
-  char *translated_text = NULL;
-  gint translation_score = 1000;
   xmlChar *prop;
   GreeterItemListItem *li;
+  char *translated_text  = NULL;
+  gint translation_score = 1000;
   
   prop = xmlGetProp (node,(const xmlChar *) "id");
   
@@ -1550,6 +1604,8 @@ parse_items (xmlNodePtr  node,
 	      item_type = GREETER_ITEM_TYPE_ENTRY;
 	    else if (strcmp ((char *) type, "list") == 0)
 	      item_type = GREETER_ITEM_TYPE_LIST;
+	    else if (strcmp ((char *) type, "button") == 0)
+	      item_type = GREETER_ITEM_TYPE_BUTTON;
 	    else
 	      {
 		g_set_error (error,
@@ -1563,12 +1619,12 @@ parse_items (xmlNodePtr  node,
 	    info = greeter_item_info_new (parent, item_type);
 	    
 	    parse_id (child, info);
-	    if G_UNLIKELY ( ! parse_button (child, info, error))
+	    if G_UNLIKELY ( ! parse_canvasbutton (child, info, error))
 	      return FALSE;
 
 	    if (button_stack != NULL)
 	      info->my_button = button_stack->data;
-	    if (info->button)
+	    if (info->canvasbutton)
 	      button_stack = g_list_prepend (button_stack, info);
 
 	    switch (item_type)
@@ -1591,6 +1647,9 @@ parse_items (xmlNodePtr  node,
 	      case GREETER_ITEM_TYPE_LIST:
 		res = parse_list (child, info, error);
 		break;
+	      case GREETER_ITEM_TYPE_BUTTON:
+		res = parse_gtkbutton (child, info, error);
+		break;
 	      default:
 		g_set_error (error,
 			     GREETER_PARSER_ERROR,
@@ -1599,7 +1658,7 @@ parse_items (xmlNodePtr  node,
 		res = FALSE;
 	      }
 
-	    if (info->button)
+	    if (info->canvasbutton)
 	      button_stack = g_list_remove (button_stack, info);
 	    
 	    if G_UNLIKELY (!res)
@@ -1636,9 +1695,10 @@ greeter_parse (const char *file, const char *datadir,
   GreeterItemInfo *root;
   xmlDocPtr doc;
   xmlNodePtr node;
+  xmlChar *prop;
   gboolean res;
   GList *items;
-
+  
   /* FIXME: EVIL! GLOBAL! */
   g_free (file_search_path);
   file_search_path = g_strdup (datadir);
@@ -1684,6 +1744,26 @@ greeter_parse (const char *file, const char *datadir,
       return NULL;
     }
 
+  /*
+   * The gtk-theme property specifies a theme specific gtk-theme to use
+   */
+  prop = xmlGetProp (node, (const xmlChar *) "gtk-theme");
+  if (prop)
+    {
+      gchar *theme_dir;
+
+      /*
+       * It might be nice if we allowed this property to also supply a gtkrc file
+       * that could be included in the theme.  Perhaps we should check first in
+       * the theme directory for a gtkrc file by the provided name and use that
+       * if found.
+       */
+      theme_dir = g_strdup_printf ("%s/%s", gtk_rc_get_theme_dir (), (char *) prop);
+      if (g_file_test (theme_dir, G_FILE_TEST_IS_DIR))
+         gdm_set_theme ((char *) prop);
+
+      xmlFree (prop);
+    }
 
   item_hash = g_hash_table_new ((GHashFunc)greeter_info_id_hash,
 				(GEqualFunc)greeter_info_id_equal);
@@ -1694,8 +1774,10 @@ greeter_parse (const char *file, const char *datadir,
 
   /* Now we can whack the hash, we don't want to keep cached
      pixbufs around anymore */
-  g_hash_table_destroy (pixbuf_hash);
-  pixbuf_hash = NULL;
+  if (pixbuf_hash != NULL) {
+     g_hash_table_destroy (pixbuf_hash);
+     pixbuf_hash = NULL;
+  }
 
   if G_UNLIKELY (!res)
     {
